/*
 ******************************************************************************

   LibBuf allocation functions

   Copyright (c) 2002  Brian S. Julin		bri@tull.umassp.edu

  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#include <ggi/internal/buf.h>
#include <ggi/internal/buf_debug.h>


/* _buf_alloc_def_llobj defines a basic ggiBuf_t capable of use with 
 * LibBuf functions only.
 */

#define _buf_def_llobj_map \
((3 << GGI_LL_ST_SHIFT) | (5 << GGI_LL_GC_SHIFT) | (8 << GGI_LL_GA_SHIFT))

ggiBuf_t _buf_alloc_def_llobj(void) {
	struct _buf_def_llobj *buf;

	buf = calloc(1, sizeof(struct _buf_def_llobj));
	if (buf == NULL) return NULL;

	buf->ptrs[3] = &(buf->stops);
	buf->ptrs[4] = &(buf->ststate);
	buf->ptrs[5] = &(buf->gc);
	buf->ptrs[6] = &(buf->gc2);
	buf->ptrs[7] = &(buf->mathgc);
	buf->ptrs[8] = &(buf->ga);

	buf->map = _buf_def_llobj_map;

	return ((ggiBuf_t)buf);

}	/* buf_alloc_def_llobj */

void _buf_free_def_llobj(ggiBuf_t buf) {
	struct _buf_def_llobj *foo;

	foo = (struct _buf_def_llobj *)buf;

	/* This is all just sanity checking.
	 * It can be disabled for performance/size if desired. 
	 */
	LIB_ASSERT(foo);
	LIB_ASSERT(foo->map == _buf_def_llobj_map);

#define BUF_DEF_LLOBJ_ASSERT(loc, field) \
LIB_ASSERT((void *)(foo->ptrs[loc])-(void *)foo == \
(void *)&(foo->field)-(void *)foo);

	BUF_DEF_LLOBJ_ASSERT(3, stops);
	BUF_DEF_LLOBJ_ASSERT(4, ststate);
	BUF_DEF_LLOBJ_ASSERT(5, gc);
	BUF_DEF_LLOBJ_ASSERT(6, gc2);
	BUF_DEF_LLOBJ_ASSERT(7, mathgc);
	BUF_DEF_LLOBJ_ASSERT(8, ga);

	free(foo);
}	/* buf_free_def_llobj */


/* TODO: functions to generate/deallocate trans-lib portable llobjs */
